#!/usr/bin/env python3

# A BornAgain auxiliary script.
# License: Public Domain.

# Checks whether a given Python3 module is installed.
#
# Takes module name as argument.
# Returns 0 if module can be imported. Otherwise returns 1.

import sys

pkg = sys.argv[1]
try:
    __import__(pkg)
except:
    print("     find_pkg.py: package", pkg, "not found")
    sys.exit(1)

print("     find_pkg.py: package", pkg, "found")
sys.exit(0)



#!/usr/bin/env python3

helptext = """
Usage: rename-example section name_in name_out

where 'section' is one of specular, scatter2d, ...
      'name' is without extension

Writes shell commands to stdout
"""

import sys

if __name__ == '__main__':
    if len(sys.argv)!=4:
        print(helptext)
        exit(1)

    s = sys.argv[1]  # section
    n0 = sys.argv[2] # name_in
    n1 = sys.argv[3] # name_out
    f0 = s+"/"+n0
    f1 = s+"/"+n1

    print(f"""\
cd /G/sw/ba
k mv rawEx/{f0}.py rawEx/{f1}.py
k mv auto/Examples/{f0}.py auto/Examples/{f1}.py
k mv auto/FigExamples/{f0}.py auto/FigExamples/{f1}.py
k mv auto/MiniExamples/{f0}.py auto/MiniExamples/{f1}.py
k mv Tests/ReferenceData/MiniExamples/{f0}.int Tests/ReferenceData/MiniExamples/{f1}.int
rm hugo/static/py/auto/Examples/{f0}.py
rm hugo/static/img/auto/{f0}.png
ed -s Tests/Examples/CMakeLists.txt <<< $'g/{n0}/s//{n1}/g\nw'
for f in `fmd`;do ed -s $f <<< $'g/{n0}/s//{1}/g\nw'; done
cd build
cm -DDEV=ON
j
t4
j excopy
j figures
cd ..
ks

kcom -m"rename example {n0} -> {n1}"
""")



#!/usr/bin/env python3
"""
Rotated pyramids on top of substrate
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, nm2


def get_sample():
    """
    A sample with rotated pyramids on top of a substrate.
    """

    # Materials
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 0.0006, 2e-08)
    substrate_color = (0.28, 0.57, 0.82)
    substrate_mat = ba.RefractiveMaterial("Substrate", substrate_color, 6e-06, 2e-08)

    # Form factors
    ff = ba.Pyramid4(10*nm, 5*nm, 54.73*deg)

    # Particles
    particle = ba.Particle(particle_mat, ff)
    particle_rotation = ba.RotationZ(45*deg)
    particle.rotate(particle_rotation)

    # Layers
    layer_1 = ba.Layer(ba.Vacuum())
    layer_1.deposit2D(ba.Dilute2D(0.0001/nm2, particle))
    layer_2 = ba.Layer(substrate_mat)

    # Sample
    sample = ba.Sample()
    sample.addLayer(layer_1)
    sample.addLayer(layer_2)

    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = 11
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, 0, 2*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    from bornagain import ba_check
    ba_check.persistence_test(result)



#!/usr/bin/env python3
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm


def get_sample():
    # Define materials
    hmdso_color = (0.25, 0.65, 0.35)
    hmdso_mat = ba.RefractiveMaterial("HMDSO", hmdso_color, 2.0888e-06, 1.3261e-08)
    ptfe_color = (0.93, 0.72, 0.25)
    ptfe_mat = ba.RefractiveMaterial("PTFE", ptfe_color, 5.20509e-06, 1.9694e-08)
    si_color = (0.30, 0.62, 0.86)
    si_mat = ba.RefractiveMaterial("Si", si_color, 5.7816e-06, 1.0229e-07)
    vacuum_mat = ba.Vacuum()

    # Define roughness
    autocorr_1 = ba.SelfAffineFractalModel(1.1, 0.3, 5*nm)
    autocorr_2 = ba.SelfAffineFractalModel(2.3, 0.3, 5*nm)

    transient = ba.TanhTransient()

    roughness_1 = ba.Roughness(autocorr_1, transient)
    roughness_2 = ba.Roughness(autocorr_2, transient)

    # Define layers
    layer_1 = ba.Layer(vacuum_mat)
    layer_2 = ba.Layer(hmdso_mat, 18.5*nm, roughness_1)
    layer_3 = ba.Layer(ptfe_mat, 22.1*nm, roughness_2)
    layer_4 = ba.Layer(si_mat)

    # Define sample
    sample = ba.Sample()
    sample.addLayer(layer_1)
    sample.addLayer(layer_2)
    sample.addLayer(layer_3)
    sample.addLayer(layer_4)

    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.4*deg)
    n = 11
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, -.6*deg, 1.4*deg)
    detector.setResolutionFunction(
        ba.ResolutionFunction2DGaussian(0.02*deg, 0.02*deg))
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    simulation.options().setIncludeSpecular(True)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    from bornagain import ba_check
    ba_check.persistence_test(result)



#!/usr/bin/env python3
"""
Core shell nanoparticles.
Alternative implementation using particle composition
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, R3


def get_sample():
    """
    A sample with box-shaped core-shell particles on top of air.
    """

    # Materials
    diff_color = (0.86, 0.24, 0.18)
    diff_mat = ba.RefractiveMaterial("Core", diff_color, -4e-6, 0)
    shell_color = (0.25, 0.65, 0.35)
    shell_mat = ba.RefractiveMaterial("Shell", shell_color, 1e-5, 2e-8)

    # Form factors
    ff_1 = ba.Box(12*nm, 12*nm, 7*nm)
    ff_2 = ba.Box(16*nm, 16*nm, 8*nm)

    # Particles
    core = ba.Particle(diff_mat, ff_1)
    shell = ba.Particle(shell_mat, ff_2)
    particle = ba.Compound()
    particle.addComponent(shell)
    particle.addComponent(core, R3(0, 0, 0.5*nm))

    # Sample with particles in vacuum over vacuum substrate
    layer_1 = ba.Layer(ba.Vacuum())
    layer_1.deposit2D(ba.Dilute2D(0.001, particle))
    layer_2 = ba.Layer(ba.Vacuum())

    sample = ba.Sample()
    sample.addLayer(layer_1)
    sample.addLayer(layer_2)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1e9, 0.1*nm, 0.2*deg)
    n = 200
    detector = ba.SphericalDetector(n, 0., 2*deg, n, 0., 2*deg)
    simulation = ba.ScatteringSimulation(beam, sample, detector)
    return simulation


if __name__ == '__main__':
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    plotargs = bp.parse_commandline()
    bp.plot_datafield(result, unit_aspect=1, **plotargs)
    bp.export(**plotargs)



#!/usr/bin/env python3
"""
GISAS of 10 boxes with custom morphology (IsGISAXS example 7).

Ten rectangular boxes with varying widths (2, 4, 6, 8, 10 nm repeat) and
heights (1 nm), all fixed length 2 nm, placed at specific (x,y) positions
with rotation angles 0, 10, 20, ..., 90 deg. All in vacuum (no substrate).
"""
import bornagain as ba
from bornagain import ba_plot as bp, deg, nm, R3


def get_sample():
    vacuum = ba.Vacuum()
    particle_color = (0.86, 0.24, 0.18)
    particle_mat = ba.RefractiveMaterial("Particle", particle_color, 6e-4, 2e-8)

    vacuum_layer = ba.Layer(vacuum)

    params = [
        # (width, pos_x, pos_y, rotation_deg)
        (2*nm,  0*nm,  0*nm,  0),
        (4*nm,  5*nm,  5*nm, 10),
        (6*nm, -5*nm, -5*nm, 20),
        (8*nm,  5*nm, -5*nm, 30),
        (10*nm,-5*nm,  5*nm, 40),
        (2*nm,  0*nm,  0*nm, 50),
        (4*nm,  5*nm,  5*nm, 60),
        (6*nm, -5*nm, -5*nm, 70),
        (8*nm,  5*nm, -5*nm, 80),
        (10*nm,-5*nm,  5*nm, 90),
    ]
    for width, px, py, rot in params:
        p = ba.Particle(particle_mat, ba.Box(2*nm, width, 1*nm))
        if rot != 0:
            p.rotate(ba.RotationZ(rot*deg))
        p.translate(R3(px, py, 0))
        vacuum_layer.deposit2D(ba.Dilute2D(0.001, p))

    vacuum_bottom = ba.Layer(vacuum)

    sample = ba.Sample()
    sample.addLayer(vacuum_layer)
    sample.addLayer(vacuum_bottom)
    return sample


def get_simulation(sample):
    beam = ba.Beam(1, 0.1*nm, 0.2*deg)
    n = 100
    detector = ba.SphericalDetector(n, -2*deg, 2*deg, n, 0, 2*deg)
    return ba.ScatteringSimulation(beam, sample, detector)


if __name__ == '__main__':
    sample = get_sample()
    simulation = get_simulation(sample)
    result = simulation.simulate()
    plotargs = bp.parse_commandline()
    bp.plot_datafield(result, unit_aspect=1, **plotargs)
    bp.export(**plotargs)



# Functional test: tests of IO operations with the IntensityData object

import unittest, numpy
import bornagain as ba


class IOTest(unittest.TestCase):
    """
    Test serialization of IntensityData
    """
    def setUp(self):
        self.arr = numpy.array([[0, 1, 2.], [3, 4, 5.]])
        numpy.savetxt('intensitydata.txt', self.arr)
        self.dat = ba.readData2D("intensitydata.txt")

    def test_1(self):
        print()
        print("Test: numpy array -> text file -> datafield -> npArray")
        print("input array: ", self.arr)
        print("read back as datafield: ", self.dat.flatVector())

        arr1 = self.dat.intensities()
        print("exported to NumPy ", arr1, flush=True)
        self.assertTrue(numpy.array_equal(arr1, self.arr))

    def test_2(self):
        print()
        print("Test: numpy array -> text file -> datafield -> flatVector -> numpy.array")
        print("input array: ", self.arr)
        print("read back as datafield: ", self.dat.flatVector())

        arr2 = numpy.array(self.dat.flatVector())
        arr2.shape = self.arr.shape
        print("converted to NumPy ", arr2, flush=True)
        self.assertTrue(numpy.array_equal(arr2, self.arr))

if __name__ == '__main__':
    unittest.main()
